----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    00:07:58 04/06/2010 
-- Design Name: 
-- Module Name:    EAU_Iter_c - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity EAU_Iter_c is
    Port ( rst : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           valid_in : in  STD_LOGIC;
           value_in : in  STD_LOGIC_VECTOR (24 downto 0);
           const_arg : in  STD_LOGIC_VECTOR (24 downto 0);
           valid_out : out  STD_LOGIC;
			  bit_out	: out STD_LOGIC;
           value_out : out  STD_LOGIC_VECTOR (24 downto 0));
end EAU_Iter_c;

architecture Behavioral of EAU_Iter_c is

component sp_fp_lt_cmp_8_16 is
  port (
    rdy : out STD_LOGIC; 
    operation_nd : in STD_LOGIC := 'X'; 
    clk : in STD_LOGIC := 'X'; 
    a : in STD_LOGIC_VECTOR ( 24 downto 0 ); 
    b : in STD_LOGIC_VECTOR ( 24 downto 0 ); 
    result : out STD_LOGIC_VECTOR ( 0 downto 0 ) 
  );
end component;

signal rdy_comp, op_nd_comp: std_logic;
signal lt_res: std_logic_vector(0 downto 0);

component shift_reg_2b_9c is
  port (
    clk : in STD_LOGIC := 'X'; 
    d : in STD_LOGIC_VECTOR ( 1 downto 0 ); 
    q : out STD_LOGIC_VECTOR ( 1 downto 0 ) 
  );
end component;

signal sr_in, sr_out: std_logic_vector(1 downto 0);

component sp_fp_sub_8_16 is
  port (
    rdy : out STD_LOGIC; 
    operation_nd : in STD_LOGIC := 'X'; 
    clk : in STD_LOGIC := 'X'; 
    a : in STD_LOGIC_VECTOR ( 24 downto 0 ); 
    b : in STD_LOGIC_VECTOR ( 24 downto 0 ); 
    result : out STD_LOGIC_VECTOR ( 24 downto 0 ) 
  );
end component;

signal rdy_sub, op_nd_sub: std_logic;
signal sub_res: std_logic_vector(24 downto 0);

component MUX2to1_25 is
    Port ( sel : in  STD_LOGIC;
           A : in  STD_LOGIC_VECTOR (24 downto 0);
           B : in  STD_LOGIC_VECTOR (24 downto 0);
           RES : out  STD_LOGIC_VECTOR (24 downto 0));
end component;

component shift_reg_25b_11c is
  port (
    clk : in STD_LOGIC := 'X'; 
    d : in STD_LOGIC_VECTOR ( 24 downto 0 ); 
    q : out STD_LOGIC_VECTOR ( 24 downto 0 ) 
  );
end component;

signal value_in_reg : std_logic_Vector(24 downto 0);

begin

op_nd_comp<=valid_in;

COMPARE_LESS_THAN: sp_fp_lt_cmp_8_16 Port Map
(
    rdy => rdy_comp, 
    operation_nd => op_nd_comp, 
    clk => clk, 
    a => const_arg, 
    b => value_in, 
    result => lt_res 
  );

sr_in(1)<=rdy_comp;
sr_in(0)<=lt_res(0);

bit_out<= sr_out(0);

SHIFT_REGISTER_1: shift_reg_2b_9c Port Map
(
    clk => clk, 
    d => sr_in, 
    q => sr_out 
  );

op_nd_sub<=valid_in;
  
SUBTRACTER: sp_fp_sub_8_16 Port Map
(
    rdy => rdy_sub, 
    operation_nd => op_nd_sub, 
    clk => clk, 
    a => value_in, 
    b => const_arg, 
    result => sub_res 
  );
  
  valid_out<=rdy_sub;

SHIFT_REGISTER_2: shift_reg_25b_11c Port Map
 (
    clk => clk,
    d => value_in, 
    q => value_in_reg 
  );

MULTIPLEXER:  MUX2to1_25 Port Map
( sel => sr_out(0),
  A => value_in_reg,
  B => sub_res,
  RES => value_out); 
  
  
  
end Behavioral;

